# Nginx 发布的流水线
# 不能删
#!groovy
pipeline {

    agent any

    parameters{
        choice(
          name: 'Operation', 
          choices: ['deploy','rollback'], 
          description: 'deploy：发布git仓库当前最新版本 <br> rollback：回退至上个提交版本'
          )
        choice(
          name: "Module", 
          choices: ["Nginx BEFE", "Nginx App", "Nginx Core", "Nginx OPS", "Nginx Partner", "Nginx VA", "Nginx DD"], 
          description:"Nginx BEFE: 10.162.144.1、10.162.144.2<br>Nginx App: 10.162.146.3、10.162.146.10<br>Nginx Core: 10.162.136.3、10.162.136.17<br>Nginx OPS: 10.162.133.13、10.162.133.18<br>Nginx Partner: 10.162.168.133、10.162.168.135<br>Nginx VA: 10.162.168.138、10.162.168.140<br>Nginx DD: 10.162.168.139、10.162.168.141"
          )

        booleanParam(
          name: 'PIC_Double_Check', 
          defaultValue: false, 
          description: '本次发布已知会给PIC，并进行了double check'
          )

        booleanParam(
          name: 'AM_Approval', 
          defaultValue: false, 
          description: '本次发布遵循变更流程，已提交变更申请并获得审批'
          )

    }

    stages {

        stage('Prepare') {
            steps {
                echo '========Prepare========'
                script {

                    sh """
                      ls -d BankingNginxDR || git clone https://ops_bot:Qwer1234@gitlab.bke.shopee.io/bke-ops/BankingNginxDR.git
                      cd BankingNginxDR
                      git pull origin master
                    """

                    if ( params.Operation == 'rollback' ) {
                    sh """ 
                      cd BankingNginxDR
                      git reset --hard HEAD~1 
                    """
                    }

                    GIT_COMMIT = sh (script: "cd BankingNginxDR && git rev-parse --short HEAD", returnStdout: true).trim()
                    GIT_MSG = sh (script: "cd BankingNginxDR && git log --pretty=format:'%s' HEAD -1", returnStdout: true).trim()

                }
                
            }
        }

        stage('Deploy') {
            steps {
                echo '========Deploy========'
                script {

                    if ( params.PIC_Double_Check == false ) {
                            echo 'PIPELINE ABORTED, NEED PIC DOUBLE CHECK'
                            currentBuild.result = 'ABORTED'
                            return
                    }

                    if ( params.AM_Approval == false ) {
                            echo 'PIPELINE ABORTED, SHOULD SEND APPLY EMAIL FIRST AND GET AM APPROVAL'
                            currentBuild.result = 'ABORTED'
                            return
                    }

                    if ( params.Module == "Nginx BEFE" ) {
                    sh """
                      cd BankingNginxDR
                      ansible-playbook -u deployop -i hosts --become --become-user=root Nginx-Biz-BEFE-Terminal.yml
                    """
                    }

                    else if ( params.Module == "Nginx App" ) {
                    sh """
                      cd BankingNginxDR
                      ansible-playbook -u deployop -i hosts --become --become-user=root Nginx-App-Terminal.yml
                    """
                    }

                    else if ( params.Module == "Nginx Core" ) {
                    sh """
                      cd BankingNginxDR
                      ansible-playbook -u deployop -i hosts --become --become-user=root Nginx-Core-Terminal.yml
                    """
                    }

                    else if ( params.Module == "Nginx OPS" ) {
                    sh """
                      cd BankingNginxDR
                      ansible-playbook -u deployop -i hosts --become --become-user=root Nginx-Ops-Terminal.yml
                    """
                    }

                    else if ( params.Module == "Nginx Partner" ) {
                    sh """
                      cd BankingNginxDR
                      ansible-playbook -u deployop -i hosts --become --become-user=root Nginx-Partner-Terminal.yml
                    """
                    }

                    else if ( params.Module == "Nginx VA" ) {
                    sh """
                      cd BankingNginxDR
                      ansible-playbook -u deployop -i hosts --become --become-user=root Nginx-VA-Terminal.yml
                    """
                    }

                    else if ( params.Module == "Nginx DD" ) {
                    sh """
                      cd BankingNginxDR
                      ansible-playbook -u deployop -i hosts --become --become-user=root Nginx-DD-Terminal.yml
                    """
                    }

                    else {
                    sh """
                      echo "Error"
                    """
                    }
                    
                }
                
            }
        }
    }

    post {
        always{
            wrap([$class: 'BuildUser']) {
            buildName "${currentBuild.number}-DR-${BUILD_USER}"
            buildDescription "<div><li>branch: master</li><li>module: <strong><span style=\"color:green;font-size: 200%\">${params.Module}</span></strong></li><li>commit: <strong><span style=\"color:blue;font-size: 200%\">${GIT_COMMIT}</span></strong></li><li>message: ${GIT_MSG}</li> "
            }
        }
    }
    
    
}
