# 远程日志的文件

```
#!groovy
pipeline{
    agent{
        label "slave"
    }
    parameters{
        choice(name: 'ENVIRONMENT', choices: ['staging','dr','third'], description: '选择环境信息')
        booleanParam(defaultValue: false, name: "TAG_RELEASE", description: '选择是否使用tag')
        gitParameter(name: 'TAG', type: 'PT_TAG', defaultValue: 'release-nonexist', sortMode: 'DESCENDING_SMART',quickFilterEnabled: true, description: 'tag列表')
        gitParameter(name: 'BRANCH', type: 'PT_BRANCH', defaultValue: 'deploy', sortMode: 'DESCENDING_SMART',quickFilterEnabled: true, description: '分之列表')
    }

    environment{
        APP_NAME = "remote-log-checker"
        VERSION = ""
        APP_PATH = "/home/deployop/${params.ENVIRONMENT}/${APP_NAME}/target/${APP_NAME}"
        GIT_BRANCH = getFinalBranch()
        staging = "remotelog-nonlive-vm1,remotelog-nonlive-vm2"
        dr = "remotelog-dr-1,remotelog-dr-2"
        third = "remotelog-third-1,remotelog-third-2"
    }

    stages{
        stage("Set Config"){
            steps{
                echo "========set config========"
                wrap([$class: 'BuildUser']) {
                    buildName "${currentBuild.number}-${params.ENVIRONMENT}-${BUILD_USER}"
                    buildDescription "branch: ${GIT_BRANCH}\ncommit: ${GIT_COMMIT}\n"
                }
            }
        }

        stage("Git Clone"){
            steps{
                echo "========fetch from repo========"
                checkout(
                    [
                        $class: 'GitSCM',
                        branches: [[
                            name: getFinalBranch()
                        ]],
                        doGenerateSubmoduleConfigurations: false,
                        extensions: [],
                        gitTool: 'Default',
                        userRemoteConfigs: [[
                            url: scm.getUserRemoteConfigs()[0].getUrl(),
                            credentialsId: scm.getUserRemoteConfigs()[0].getCredentialsId()
                        ]],
                    ]
                )
            }
        }

    stage('Build'){
        steps{
              withMaven(
                    // Maven installation declared in the Jenkins "Global Tool Configuration"
                    maven: 'maven3.6.3', // (1)
                    // Use `$WORKSPACE/.repository` for local repository folder to avoid shared repositories
                    // mavenLocalRepo: '.repository', // (2)
                    // Maven settings.xml file defined with the Jenkins Config File Provider Plugin
                    // We recommend to define Maven settings.xml globally at the folder level using
                    // navigating to the folder configuration in the section "Pipeline Maven Configuration / Override global Maven configuration"
                    // or globally to the entire master navigating to  "Manage Jenkins / Global Tools Configuration"
                    mavenSettingsConfig: 'maven-settings' // (3)
                ) {
                    sh "mvn clean package -Dmaven.test.skip=true -Denv=${params.ENVIRONMENT}"
            }
        }
    }

        stage("Deploy"){
            steps{
                echo "========deploy to the target server========"
                script {
                    def config = ""

                    switch(params.ENVIRONMENT){
                        case "staging":
                            config = "${ staging }"
                            break
                        case "dr":
                            config = "${ dr }"
                            break
                        default:
                            config = "${ third }"
                            break
                    }

                    config.tokenize(',').each {
                        deploy("${ it }")
                    }
                }
            }
        }
    }
}

// 处理函数

def getFinalBranch() {
    // 反回git分支信息
    def tag = ""
    if (params.TAG_RELEASE){
        tag = params.TAG
    }else {
        tag = params.BRANCH
    }
    return tag
}

def transfer2SingleDir(String tag) {
    String tmp = tag.replaceAll("origin/", "")
    return tmp.replaceAll("/", "-")
}

def deploy(String ServerName) {
    def tag = getFinalBranch()
    def finalTag = transfer2SingleDir(tag)
    sshPublisher(
        publishers: [
            sshPublisherDesc(
                configName: "${ ServerName }",
                verbose: true,
                transfers: [
                    sshTransfer(
                        cleanRemote: true,
                        excludes: '',
                        execCommand: "sudo echo -e '[program:${APP_NAME}-${params.ENVIRONMENT}] \ncommand=/bin/bash /home/deployop/${params.ENVIRONMENT}/${APP_NAME}/scripts/stop-vm.sh ${params.ENVIRONMENT}  \ncommand=/bin/bash  /home/deployop/${params.ENVIRONMENT}/${ APP_NAME }/scripts/start-vm.sh ${params.ENVIRONMENT} \nautostart=true \nautorestart=true \nuser=deployop \nstartsecs=10 \nkillasgroup=true \nstopasgroup=true \nredirect_stderr=true \nstdout_logfile=/home/deployop/${params.ENVIRONMENT}/${ APP_NAME }/log/log.log' > /etc/supervisor/conf.d/${APP_NAME}-${params.ENVIRONMENT}.conf;chmod +x ${params.ENVIRONMENT}/${ APP_NAME }/scripts/*.sh;mv /home/deployop/${params.ENVIRONMENT}/${APP_NAME}/target/*.jar /home/deployop/${params.ENVIRONMENT}/${APP_NAME}/  ;mkdir -p /home/deployop/${params.ENVIRONMENT}/${APP_NAME}/log/;sudo supervisorctl update;sudo supervisorctl restart ${APP_NAME}-${params.ENVIRONMENT}",
                        execTimeout: 120000,
                        flatten: false,
                        makeEmptyDirs: false,
                        noDefaultExcludes: false,
                        patternSeparator: '[, ]+',
                        remoteDirectory: "${params.ENVIRONMENT}/${ APP_NAME }/",
                        remoteDirectorySDF: false,
                        removePrefix: '', 
                        sourceFiles: '**/*'
                       
                    )
                ],
            )
        ]
    )
}




```