from fastapi import FastAPI, APIRouter, Path
from pydantic import BaseModel
from typing import List

app = FastAPI()
todo_rotour = APIRouter()


class TodoItem(BaseModel):
    item: str
    status: bool

    class Config:
        schema_extra = {
            "example": {
                "item": "代办任务名称",
                "status": False
            }
        }


class Todo(BaseModel):
    id: int
    item: TodoItem


# 创建数据库
todo_list: List[Todo] = []


# 创建路由
@todo_rotour.post("/todo")
async def todo_add(todo: Todo) -> dict:
    todo_list.append(todo)
    return {
        "todo": todo
    }


@todo_rotour.get("/todo")
async def todo_get_list() -> dict:
    return {
        "data": todo_list
    }


@todo_rotour.get("/todo/{todo_id}")
async def todo_get_by_id(todo_id: int = Path(..., title="任务ID")) -> dict:
    # 查找并反回
    for tt in todo_list:
        if tt.id == todo_id:
            return {
                "data": tt
            }
    # 查不到，返回空
    return {
        "data": None
    }


@todo_rotour.put("/todo/{todo_id}")
async def todo_update(todo_id: int = Path(..., title="任务ID"), todo: Todo = None) -> dict:
    # 查找替换
    for i in range(len(todo_list)):
        if todo_list[i].id == todo_id:
            todo_list[i] = todo
            return {
                "data": todo_list[i]
            }
    # 查找失败
    return {
        "data": None
    }


@todo_rotour.delete("/todo/{todo_id}")
async def todo_delete_by_id(todo_id: int = Path(..., title="任务ID")) -> dict:
    # 查找并删除
    for i in range(len(todo_list)):
        if todo_list[i].id == todo_id:
            return {
                "data": todo_list.pop(i)
            }
    # 删除失败
    return {
        "data": None
    }


# 挂载路由
app.include_router(todo_rotour)

# 启动应用
if __name__ == "__main__":
    import uvicorn

    uvicorn.run("main:app", host="127.0.0.1", port=8888, reload=True)
