# -*- coding: utf-8 -*-
# !/usr/bin/env python3
# yule.wen 2021/7/8-3:04
# use func:


import json
import datetime
import sys
import os
import subprocess

help_str = """
-c create single bucket
-o the bucket owner,this arg is must need 
-f create bucket from file
-b build auth policy
-u user, auth the bucket to the user 
-p policy eg: read/write/all

1.create single bucket
python3 ceph-create.py -c bucket_name -o bke_log

2.create bucket from bucket_file
python3 ceph-create.py -f bucket_file_name -o bke_log

3.auth bucket to user 
python3 ceph-create.py -b bucket_name -o bke_log -u bke_core -p [read|write|all]


"""

ak_sk = {
    "ak_bke_mobile": "AA4SBB0XBOV04SZGFB3Y",
    "sk_bke_mobile": "Ydw5yZojMCFBphApSRbmdS90zckQCahvHYjNCzpo",
    "ak_bke_core": "OJ3E2V5JOTD7E4P1LFUT",
    "sk_bke_core": "mzUEHjyA0yWNjBPCO7SNXhgHhP5GJyHFn8S8hxrO",
    "ak_bke_dwh": "RQ9OFBNPLWP2L7AIW6V0",
    "sk_bke_dwh": "AYADmM1mhgIZrVY64iDu4eKREFSOwwiKqsrJO4fj",
    "ak_banking_toc": "H3F7BZ0A7TD04YGRWDAW",
    "sk_banking_toc": "UBdNoP6FKi1RJJlsnUEfa1wty2pgh8bzTTYtj0tJ",
    "ak_tracker": "9HJH0I2D87LTYO6M4OQB",
    "sk_tracker": "YRX0kkJrmB11lEZs1E8SivlRvgxQeilWYveBxxvh",
    "ak_bke_log": "9BWV7SPDG3DK54MRA41C",
    "sk_bke_log": "k4VqAHLZhXwxolKBlQA0SHiwXbPcMSUXd67xqsev",
}


def print_green(s):
    print('\033[0;32;40m %s \033[0m' % s)


def print_red(s):
    print('\033[0;31;40m %s \033[0m' % s)


def exec_func(exec_str):
    res = subprocess.Popen(exec_str, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    print(str(res.stdout.read(), encoding='utf-8'))


def build_police(bucket, owner, user, policy):
    date_obj = datetime.datetime.now()
    date_str = "%s-%s-%s_%s%s%s" % (
        date_obj.year, date_obj.month, date_obj.day, date_obj.hour, date_obj.month, date_obj.second)

    bucket = bucket.strip('')
    owner = owner.strip('')
    user = user.strip('')
    policy = policy.strip()
    ak_key = ak_sk.get('ak_' + owner)
    sk_key = ak_sk.get('sk_' + owner)

    def police_template(auth_user, auth_action, auth_bucket):
        return {
            "Sid": auth_user,
            "Effect": "Allow",
            "Principal": {"AWS": "arn:aws:iam:::user/" + auth_user},
            "Action": auth_action,
            "Resource": [
                "arn:aws:s3:::" + auth_bucket + "/*",
                "arn:aws:s3:::" + auth_bucket,
            ]
        }

    statements = []

    def auth_temp(auth_policy):
        if auth_policy == "read":
            statements.append(police_template(user, "s3:GetObject*", bucket))
            statements.append(police_template(user, "s3:ListBucket*", bucket))
        elif auth_policy == "write":
            statements.append(police_template(user, "s3:DeleteObject*", bucket))
            statements.append(police_template(user, "s3:PutObject*", bucket))
        elif auth_policy == "all":
            statements.append(police_template(user, "s3:*", bucket))
        else:
            print_red("auth error")
            exit(1)
        return {"Version": date_str, "Statement": statements}

    police = auth_temp(policy)
    from pathlib import Path
    dir = Path('./policy_record')
    if not dir.exists():
        os.mkdir('policy_record')
    police_save_filename = "./policy_record/%s_policy_%s.json" % (bucket, date_str)
    with open(police_save_filename, 'a+') as f:
        f.write(json.dumps(police))

        exec_str = 's3cmd --no-ssl --access_key=%s --secret_key=%s --host-bucket= ---host=127.0.0.1:7480 ' \
                   'setpolicy' \
                   ' %s ' \
                   's3://%s' % (ak_key, sk_key, police_save_filename, bucket)
        print(exec_str)
        print(exec_func(exec_str))


def create_bucket(create_bucket_name, bucket_owner):
    create_bucket_name = create_bucket_name.strip('')
    bucket_owner = bucket_owner.strip('')
    if 'ak_' + bucket_owner not in ak_sk:
        print_red('the owner is error')
        exit(1)
    ak = ak_sk.get('ak_' + bucket_owner)
    sk = ak_sk.get('sk_' + bucket_owner)

    exec_str = 's3cmd --no-ssl --access_key=%s --secret_key=%s --host-bucket= ' \
               '--host=127.0.0.1:7480 mb s3://%s' % (ak, sk, create_bucket_name)
    print(exec_str)
    print(exec_func(exec_str))


args_list = sys.argv
args_lens = len(args_list)

action = ''
name = ''
print(args_list, args_lens)


if args_lens == 5 or args_lens == 9:
    action = args_list[1]
    if action == '-c':
        bucket_name = args_list[2]
        owner = args_list[4]
        create_bucket(bucket_name, owner)
    elif action == '-f':
        bucket_file_name = args_list[2]
        owner = args_list[4]
        print(bucket_file_name, owner)
        with open('./%s' % bucket_file_name, 'r') as f:
            bucket_name_list = f.readlines()
        for i in bucket_name_list:
            create_bucket(i, owner)

    elif action == '-b':
        bucket_name = args_list[2]
        owner = args_list[4]
        user = args_list[6]
        policy = args_list[8]
        build_police(bucket_name, owner, user, policy)
    else:
        print_red('plz input correct')
        print_green(help_str)
else:
    print_red("plz input correct!")
    print_green(help_str)
    exit(1)
