#!/usr/bin/python3
# yule.wen 2021/8/11
# -*- coding: utf-8 -*-
# !/usr/bin/env python3
# yule.wen 2021/7/8-3:04
# use func:


import json
import datetime
import sys
import os
import subprocess
from boto3.session import Session

help_str = """
-c create single bucket
-o the bucket owner,this arg is must need
-f create bucket from file
-b build auth policy
-u user, auth the bucket to the user
-p policy eg: read/write/all

1.create single bucket
python3 ceph-bucket-manage.py -c bucket_name -o bke_log

2.create bucket from bucket_file
python3 ceph-bucket-manage.py -f bucket_file_name -o bke_log

3.auth bucket to user
python3 ceph-bucket-manage.py -b bucket_name -o bke_log -u bke_core -p [read|write|all]


"""


def get_role_list():
    cmd_get_role_list = 'radosgw-admin user list'
    res_role_list = os.popen(cmd_get_role_list)
    role_list = json.load(res_role_list)
    return role_list


def get_role_key(r_list):
    all_info = {}
    for role in r_list:
        cmd = 'radosgw-admin user info --uid=%s' % role
        role_key = json.load(os.popen(cmd)).get("keys")
        all_info[role] = role_key
    return all_info


def get_bucket_policy(bucket, owner):
    roles_list = get_role_list()
    all_role_key = get_role_key(roles_list)
    access_key = all_role_key.get(owner)[0].get("access_key")
    secret_key = all_role_key.get(owner)[0].get("secret_key")
    url = "http://127.0.0.1:7480"

    session = Session(access_key, secret_key)
    s3_client = session.client('s3', endpoint_url=url)
    try:
        result = s3_client.get_bucket_policy(Bucket=bucket)
    except Exception as e:
        print("The bucket policy does not exist")
        return {}
    policy = json.loads(result['Policy'])

    if policy:
        return policy
    else:
        print("The bucket has no policy!\n New authorization matrix will be created！ ")
        return policy


roles_list = get_role_list()
ak_sk = get_role_key(roles_list)


def print_green(s):
    print('\033[0;32;40m %s \033[0m' % s)


def print_red(s):
    print('\033[0;31;40m %s \033[0m' % s)


def exec_func(exec_str):
    res = subprocess.Popen(exec_str, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    print(str(res.stdout.read(), encoding='utf-8'))


def build_police(bucket, arg_owner, arg_user, policy):
    date_obj = datetime.datetime.now()
    date_str = "%s-%s-%s_%s%s%s" % (
        date_obj.year, date_obj.month, date_obj.day, date_obj.hour, date_obj.month, date_obj.second)

    bucket = bucket.strip('')
    owner = arg_owner.strip('')
    user = arg_user.strip('')
    policy = policy.strip()

    if not ak_sk.get(owner):
        print_red('the owner is error')
        exit(1)
    ak_key = ak_sk.get(owner)[0].get("access_key")
    sk_key = ak_sk.get(owner)[0].get("secret_key")

    def police_template(auth_user, auth_action, auth_bucket):
        return {
            "Sid": auth_user + '.' + auth_action,
            "Effect": "Allow",
            "Principal": {"AWS": "arn:aws:iam:::user/" + auth_user},
            "Action": auth_action,
            "Resource": [
                "arn:aws:s3:::" + auth_bucket + "/*",
                "arn:aws:s3:::" + auth_bucket,
            ]
        }

    statements = []

    def auth_temp(auth_policy_type):
        if auth_policy_type == "read":
            statements.append(police_template(user, "s3:GetObject*", bucket))
            statements.append(police_template(user, "s3:ListBucket*", bucket))
        elif auth_policy_type == "write":
            statements.append(police_template(user, "s3:DeleteObject*", bucket))
            statements.append(police_template(user, "s3:PutObject*", bucket))
        elif auth_policy_type == "all":
            statements.append(police_template(user, "s3:*", bucket))
        else:
            print_red("auth error")
            exit(1)
        return statements

    policy_list = auth_temp(policy)

    old_policy = get_bucket_policy(bucket, owner)

    if old_policy:
        old_list = old_policy.get("Statement")
        for i in policy_list:
            old_list.append(i)
        police = {"Version": "2012-10-17", "Statement": old_list}
    else:
        police = {"Version": "2012-10-17", "Statement": policy_list}

    from pathlib import Path
    dir = Path('./policy_record')
    if not dir.exists():
        os.mkdir('policy_record')
    police_save_filename = "./policy_record/%s_policy_%s_%s.json" % (bucket, policy, date_str)

    with open(police_save_filename, 'a+') as f:
        f.write(json.dumps(police))
        exec_str = "s3cmd --no-ssl --access_key=%s --secret_key=%s --host-bucket= --host=127.0.0.1:7480 setpolicy %s " \
                   "s3://%s" % (
                       ak_key, sk_key, police_save_filename, bucket)
    # print(exec_str)
    res = subprocess.Popen(exec_str, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    print(str(res.stdout.read(), encoding='utf-8'))
    print_green("auth policy success")


def create_bucket(create_bucket_name, bucket_owner):
    create_bucket_name = create_bucket_name.strip('')
    # bucket_owner = bucket_owner
    # print(bucket_owner)
    if bucket_owner not in ak_sk:
        print_red('the owner is error')
        exit(1)
    ak = ak_sk.get(bucket_owner)[0].get("access_key")
    sk = ak_sk.get(bucket_owner)[0].get("secret_key")

    exec_str = 's3cmd --no-ssl --access_key=%s --secret_key=%s --host-bucket= ' \
               '--host=127.0.0.1:7480 mb s3://%s' % (ak, sk, create_bucket_name)
    # print("execute cmd is :")
    # print(exec_str)
    res = subprocess.Popen(exec_str, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    print(str(res.stdout.read(), encoding='utf-8'))



args_list = sys.argv
args_lens = len(args_list)

action = ''
name = ''

if args_lens == 5 or args_lens == 9:
    action = args_list[1]
    if action == '-c':
        bucket_name = args_list[2]
        owner = args_list[4]
        create_bucket(bucket_name, owner)
    elif action == '-f':
        bucket_file_name = args_list[2]
        owner = args_list[4]
        print(bucket_file_name, owner)
        with open('./%s' % bucket_file_name, 'r') as f:
            bucket_name_list = f.readlines()
        for i in bucket_name_list:
            create_bucket(i, owner)

    elif action == '-b':
        bucket_name = args_list[2]
        owner = args_list[4]
        user = args_list[6]
        policy = args_list[8]
        build_police(bucket_name, owner, user, policy)
    else:
        print_red('plz input correct')
        print_green(help_str)
else:
    print_red("plz input correct!")
    print_green(help_str)
    exit(1)
