import json
import datetime
import requests
import click
import sys
import re

cookies = {
    # 日志平台token(这个是ph bank的nonlive环境日志平台的token, 其它地区的日志平台需要找tao.yang@shopee.com申请，参考https://jira.shopee.io/projects/SPPSP/queues/issue/SPPSP-7146)
    'infra_platform_token': 'eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJlbWFpbCI6ImppbnFpdS5odWFuZ0BzaG9wZWUuY29tIiwibmFtZSI6IkppbnFpdSjpu4Tph5Hnp4spIiwicGljdHVyZSI6IiIsIlJvbGUiOjIsImV4cCI6MTY5NzMwMjY0NywiaWF0IjoxNjY2MDkwNjQ3fQ.JdXF7GiJhZub061_f0nBPO3kt2nviPghDiHFBE18W3c',
}

headers = {
    'accept': 'application/json, text/plain, */*',
    'token': 'RELEASE-X87OKRCKW6732DFD',
}

region_key_map = {
    'ph': 'ph1',
    'id': 'id3',
    'sg': 'sg1',
    'my': 'ph1'
}

log_platform_url_key_map = {
    'ph_nonlive': 'https://log-nonlive.uat.ph.seabank.io',
    'ph_live': 'https://log.ph.seabank.io',
    'id_nonlive': 'https://log-nonlive.bke.shopee.io/',
    'id_live': '',
    'my_nonlive': 'https://log-nonlive.uat.ph.seabank.io'
}

cid_key_map = {
    'ph': 'ph',
    'sg': 'sg',
    'id': 'id',
    'my': 'ph'
}

release_platform_url_key_map = {
    'my': 'https://my-release.ph.seabank.io',
    'ph': 'https://release.ph.seabank.io',
    'id': 'https://release.id.seabank.io',
    'sg': 'https://release.sg.maribank.io',
}

env_key_map = {
    'nonlive': "dev[1-9]|sit[1-9]|uat[1-9]|staging",
    'live': "live",
    'dr': "dr"
}


def fetch_relation_env_project_k8s_list(cid):
    release_platform_url = release_platform_url_key_map[cid]
    params = {
        'pageNo': '1',
        'pageSize': '1000',
    }

    response = requests.get(release_platform_url+'/api/v1/app/relation_env_project_k8s_list', params=params, headers=headers)

    return json.loads(response.content)


def fetch_app_list(cid):
    release_platform_url = release_platform_url_key_map[cid]
    params = {
        'pageNo': '1',
        'pageSize': '1000',
        'deploy_type': 'k8s',
    }
    response = requests.get(release_platform_url+'/api/v1/app/app_list', params=params, headers=headers)
    return json.loads(response.content)


def fetch_single_app(cid, name):
    release_platform_url = release_platform_url_key_map[cid]
    params = {
        'pageNo': '1',
        'pageSize': '1000',
        'deploy_type': 'k8s',
        'app_name': name,
    }
    response = requests.get(release_platform_url+'/api/v1/app/app_list', params=params, headers=headers)
    return json.loads(response.content)


# create logstore
def create_logstore(appname, env, k8s_namespace_name, project_name, log_platform_url, cid, idc):
    formattedappname = appname.replace("-", "")
    zone = k8s_namespace_name.split("-")[0]
    region = k8s_namespace_name.split("-")[2]
    daily_size_gigabytes = 100
    log_retention_apply_reason = '1'
    retention_days = 10
    json_data = {
        'dockings': [
            {
                'configurations': {
                    'basic_info': {
                        'project_name': project_name,
                        'module': formattedappname,
                        'env': env,
                        'cid': cid,
                        'idc': idc,
                        'log_name': project_name+"."+formattedappname+"-"+env+"-"+cid+"_"+idc,
                    },
                    'resource_configuration': {
                        'daily_size_gigabytes': daily_size_gigabytes,
                        'retention_days': retention_days,
                        'log_retention_apply_reason': log_retention_apply_reason,
                    },
                    'log_config': {
                        'log_name': project_name+"."+formattedappname+"-"+env+"-"+cid+"_"+idc,
                        'sources': {
                            project_name+"."+formattedappname+"-"+env+"-"+cid+"_"+idc+'_source0': {
                                'name': project_name+"."+formattedappname+"-"+env+"-"+cid+"_"+idc+'_source0',
                                'source_type': 0,
                                'enable': True,
                                'extract_config': {
                                    'extractor_type': 0,
                                    'pattern': '',
                                },
                                'standard_file_config': {
                                    'collecting_paths': [
                                        {
                                            'pattern': '/data/log/'+zone+"-"+formattedappname+"-"+env+"-"+region+'/*.log',
                                            'file_severity': 'INFO',
                                        },
                                        {
                                            'pattern': '/data/log/'+zone+"-"+formattedappname+"-"+env+"-"+region+'/*/*.log',
                                            'file_severity': 'INFO',
                                        },
                                    ],
                                    'deployment_type': 'DeploymentTypeBareMetal',
                                    'labels': [],
                                    'collecting_path_pattern': [],
                                },
                                'timezone_offset_sec': 28800,
                            },
                        },
                        'pr_es_config': {
                            'with_file_info': True,
                        },
                        'chan_size': 0,
                        'batch_size': 0,
                        'batch_wait': 0,
                    },
                },
            },
        ],
    }
    now = datetime.datetime.now()
    now_time = now.strftime("%Y-%m-%d %H:%M:%S")

    response = requests.post(log_platform_url+'/v1/docking/dockings:batch', cookies=cookies, json=json_data)

    json_data = json.loads(response.content)
    if 'dockings' in json_data:
        print(now_time+"[Created done]---"+project_name+"."+formattedappname+"-"+env+"-"+cid+"_"+idc)
    else:
        print(now_time+"[Created fail]---"+project_name+"."+formattedappname+"-"+env+"-"+cid+"_"+idc+" :reason is : "+json.dumps(json_data))


# fetch approving logstore
def fetch_approving_status_log(project_name, log_platform_url):
    params = {
        'project_name': project_name,
        'view': 'WITHOUT_STATUS_DONE',
    }

    response = requests.get(log_platform_url+'/v1/docking/dockings', params=params, cookies=cookies)

    return json.loads(response.content)


# approve logstore
def approve_logstore(logid, log_platform_url):
    json_data = {
        'id': logid,
    }

    response = requests.post(log_platform_url+'/v1/docking/dockings/'+logid+':approve', cookies=cookies, json=json_data)

    now = datetime.datetime.now()
    now_time = now.strftime("%Y-%m-%d %H:%M:%S")
    json_data = json.loads(response.content)
    if 'code' not in json_data:
        print(now_time+"[Approved done]---"+logid)
    elif json_data['code'] == 3:
        print(now_time+"[Approved already]---"+logid)
    else:
        print(now_time+"[Approved fail]---"+logid+json.dumps(json_data))


def fetch_existed_logname(project_name, log_platform_url):
    params = {
        'project_name': project_name,
        'view': 'WITH_ALL_STATUS',
    }

    response = requests.get(log_platform_url+'/v1/docking/dockings', params=params, cookies=cookies)

    json_data = json.loads(response.content)
    return json_data


def compare_two_str(existed_logname, new_logname):
    for logname in existed_logname['dockings']:
        tmp_logname = logname['configurations']['basic_info']['log_name']
        if new_logname == tmp_logname:
            return True
    return False


def logstore_create_batch_for_sure(project, cid, env):
    project_name = project
    env = env
    region = cid
    cid = cid_key_map[cid]
    idc = region_key_map[cid]
    env_regex = env_key_map[env]
    new_app_list = []

    log_platform_url = log_platform_url_key_map[cid + "_" + env]
    # Check whether logstore exists or not
    existed_logname = fetch_existed_logname(project_name, log_platform_url)
    # Create logstore
    relation_env_project_k8s_list = fetch_relation_env_project_k8s_list(region)
    app_list = fetch_app_list(region)
    if app_list['code'] != 0:
        print("Failed to fetch app_list info")
    if relation_env_project_k8s_list['code'] == 0:
        for env in relation_env_project_k8s_list['data']['data']:
            for app in app_list['data']['data']:
                if app['project_name'] == env['project_name']:
                    # If logstore exists, then jump
                    appname = app['app_name'].replace("-", "")
                    # 因为日志平台有分live,nonlive,dr，所以从发布系统获取环境信息，还得判断过滤一下
                    if not re.match(env_regex, env['env_name']):
                        continue
                    new_logname = project_name + "." + appname + "-" + env['env_name'] + "-" + cid + "_" + idc
                    if compare_two_str(existed_logname, new_logname):
                        continue
                    else:
                        new_app_list.append(new_logname)

    return new_app_list


def logstore_create_batch(project, cid, env):
    # 让用户确认是否继续创建
    new_app_list = logstore_create_batch_for_sure(project, cid, env)
    if len(new_app_list) == 0:
        print("No new application found in release platform")
        sys.exit(1)
    print("The following logstore(s) will be created")
    print(new_app_list)
    answer = input("The following logstore(s) will be created, are you sure(yes/no)?")
    if answer != "yes":
        sys.exit(1)

    # 用户确定继续创建后才继续
    project_name = project
    env = env
    region = cid
    cid = cid_key_map[cid]
    idc = region_key_map[cid]
    env_regex = env_key_map[env]

    log_platform_url = log_platform_url_key_map[cid + "_" + env]
    # Check whether logstore exists or not
    existed_logname = fetch_existed_logname(project_name, log_platform_url)

    # Create logstore
    relation_env_project_k8s_list = fetch_relation_env_project_k8s_list(region)
    app_list = fetch_app_list(region)
    if app_list['code'] != 0:
        print("Failed to fetch app_list info")
    if relation_env_project_k8s_list['code'] == 0:
        for env in relation_env_project_k8s_list['data']['data']:
            for app in app_list['data']['data']:
                if app['project_name'] == env['project_name']:
                    # If logstore exists, then jump
                    appname = app['app_name'].replace("-", "")
                    # 因为日志平台有分live,nonlive,dr，所以从发布系统获取环境信息，还得判断过滤一下
                    if not re.match(env_regex, env['env_name']):
                        continue
                    new_logname = project_name + "." + appname + "-" + env['env_name'] + "-" + cid + "_" + idc
                    if compare_two_str(existed_logname, new_logname):
                        continue
                    else:
                        create_logstore(app['app_name'], env['env_name'], env['k8s_namespace_name'], project_name, log_platform_url, cid, idc)
    else:
        print("Failed to fetch relation_env_project_k8s_list info")

    # Approve logstore
    logid_list = fetch_approving_status_log(project_name, log_platform_url)
    for logid in logid_list['dockings']:
        # 处于pending create的状态时才approve
        if logid['status'] == 'StatusPendingCreate':
            approve_logstore(logid['id'], log_platform_url)


def logstore_create_single(project, cid, env, name):
    project_name = project
    env = env
    region = cid
    cid = cid_key_map[cid]
    idc = region_key_map[cid]
    env_regex = env_key_map[env]

    log_platform_url = log_platform_url_key_map[cid + "_" + env]
    # Check whether logstore exists or not
    existed_logname = fetch_existed_logname(project_name, log_platform_url)

    # Create logstore
    relation_env_project_k8s_list = fetch_relation_env_project_k8s_list(region)
    app_list = fetch_single_app(region, name)
    if app_list['data']['totalCount'] == 0:
        print("Failed to fetch application---"+name+"---info")
        sys.exit(1)
    if relation_env_project_k8s_list['code'] == 0:
        for env in relation_env_project_k8s_list['data']['data']:
            for app in app_list['data']['data']:
                if app['project_name'] == env['project_name']:
                    # 因为发布平台是模糊搜索的，所以这里需要完全匹配应用名
                    if app['app_name'] == name:
                        # If logstore exists, then jump
                        appname = app['app_name'].replace("-", "")
                        # 因为日志平台有分live,nonlive,dr，所以从发布系统获取环境信息，还得判断过滤一下
                        if not re.match(env_regex, env['env_name']):
                            continue
                        new_logname = project_name + "." + appname + "-" + env['env_name'] + "-" + cid + "_" + idc
                        if compare_two_str(existed_logname, new_logname):
                            continue
                        else:
                            create_logstore(app['app_name'], env['env_name'], env['k8s_namespace_name'], project_name, log_platform_url, cid, idc)
        print("[Done]: application "+name+" has been created")
    else:
        print("Failed to fetch relation_env_project_k8s_list info")

    # Approve logstore
    logid_list = fetch_approving_status_log(project_name, log_platform_url)
    for logid in logid_list['dockings']:
        # 处于pending create的状态时才approve
        if logid['status'] == 'StatusPendingCreate':
            approve_logstore(logid['id'], log_platform_url)


def params_check(action, project, cid, env, name):
    if project == "" or project is None:
        print("error: project param is required")
        sys.exit(1)
    if cid == "" or cid is None:
        print("error: cid param is required")
        sys.exit(1)
    if env == "" or env is None:
        print("error: env param is required")
        sys.exit(1)
    if action == 'logstore-create-single':
        if name == "" or name is None:
            print("error: name param is required")
            sys.exit(1)


@click.command()
@click.argument('action', type=click.Choice(['logstore-create-batch', 'logstore-create-single'], case_sensitive=True))
@click.option('-p', '--project', 'project', required=True, default="",
              help='project name: ibank, corebank, mybank.etc')
@click.option('-c', '--cid', 'cid', required=True, default="",
              help='cid: ph, sg, id, my.etc')
@click.option('-e', '--env', 'env', default="", required=True,
              help='env: live, nonlive, dr.etc')
@click.option('-n', '--name', 'name', required=True, default="",
              help='appname')
def main(action, project, cid, env, name):
    # 参数校验
    params_check(action, project, cid, env, name)
    if action == "logstore-create-batch":
        logstore_create_batch(project, cid, env)
    elif action == "logstore-create-single":
        logstore_create_single(project, cid, env, name)
    else:
        print("Invalid action")


if __name__ == '__main__':
    main()