#!groovy

def GROUP='shopee' 
def PROJECT_NAME='corebke'
def CANARY_REPLICAS='2'
def REGISTRY_CRED='bke-harbor'
def kubeConfigId='dev-k8s'
def SVR_NAME='smart-om'
def APP_NAME='smart-om'
def REGION='id'
def PORT='18889'

def k8sEnv = [
        dev1             : 'dev-k8s',
        dev2             : 'dev-k8s',
        dev3             : 'dev-k8s',
        dev4             : 'dev-k8s',
        dev5             : 'dev-k8s',
        dev6             : 'dev-k8s',
        dev7             : 'dev-k8s',
        sit1             : 'dev-k8s',
        sit2             : 'dev-k8s',
        uat1             : 'core-uat-k8s',
        uat2             : 'core-uat-k8s',
        live             : 'live-core',
]


def dockerImageExists(String imageId) {
    cmd = "bash -c '(docker inspect ${imageId} > /dev/null 2>&1 && echo found) || echo notfound'"
    found = sh([returnStdout: true, script: cmd]).trim()
    if (found == 'found') {
        return true
    }
    return false
}

node ('slave'){

    properties([
        parameters([
            string(name: 'REPLICAS', defaultValue: '1', description: ''),
            string(name: 'CPU', defaultValue: '2', description: ''),
            string(name: 'MEM', defaultValue: '4', description: ''),
            choice(name: 'ENVIRONMENT', choices: ['dev1','dev2', 'dev3', 'dev4','dev5','dev6','dev7','sit1', 'sit2','uat2','live'], description: ''),
            booleanParam(defaultValue: false, name: "CANARY", description: ''),
            booleanParam(defaultValue: false, name: "TAG_RELEASE", description: ''),
            gitParameter(name: 'TAG', type: 'PT_TAG', defaultValue: 'release-nonexist', sortMode: 'DESCENDING_SMART',quickFilterEnabled: true, description: ''),
            gitParameter(name: 'BRANCH', type: 'PT_BRANCH', defaultValue: 'deploy', sortMode: 'DESCENDING_SMART',quickFilterEnabled: true, description: '')
        ])
    ])


    def tag=''
    if (params.TAG_RELEASE){
        tag = params.TAG
    }else {
        tag = params.BRANCH
    }
    echo "tag....  ${tag}"

    def scmUrl = scm.getUserRemoteConfigs()[0].getUrl()
    def scmCred = scm.getUserRemoteConfigs()[0].getCredentialsId()
    def canary = params.CANARY
    def myRepo = checkout([$class: 'GitSCM',
                           branches: [[name: "${tag}"]],
                           doGenerateSubmoduleConfigurations: false,
                           extensions: [],
                           gitTool: 'Default',
                           userRemoteConfigs: [[url: "${scmUrl}", credentialsId: "${scmCred}"
        ]],
    ])
    def gitCommit = myRepo.GIT_COMMIT
    def gitBranch = myRepo.GIT_BRANCH
    def shortGitCommit = "${gitCommit[0..10]}"
    def previousGitCommit = sh(script: "git rev-parse ${gitCommit}", returnStdout: true)
    def ENV="${params.ENVIRONMENT}"
    def NAMESPACE="${PROJECT_NAME}-${ENV}-${REGION}"

    echo "${shortGitCommit}"
    wrap([$class: 'BuildUser']) {
        buildName "${currentBuild.number}-${params.ENVIRONMENT}-${BUILD_USER}"
        buildDescription "branch: ${tag}\ncommit: ${shortGitCommit}"
    }

    def imageTag = ''
    if (params.TAG_RELEASE){
        imageTag = tag
    } else {
        imageTag = "git-${shortGitCommit}"
    }
    echo "image tag... ${imageTag}"
    def IMAG_NAME= "harbor-nonlive.bke.shopee.io/${GROUP}/${PROJECT_NAME}/${APP_NAME}:${imageTag}"





    stage('Build'){
        docker.withTool('docker in slave') {

            docker.withRegistry('https://harbor-nonlive.bke.shopee.io', "${REGISTRY_CRED}"){
                if (!dockerImageExists("${IMAG_NAME}")){
                    def customImage = docker.build("${IMAG_NAME}", "--pull --build-arg APP_NAME=${APP_NAME} --build-arg ENVIRONMENT=${ENV} --network=host -f deploy/Dockerfile .")
                    customImage.push()
                }
            }
        }
    }
    
    
    stage('Config  Deploy file'){
                  
                def env="${params.ENVIRONMENT}"
                def projectName="${PROJECT_NAME}"
                def cid="${REGION}"
                def DEPLOY_NAMESPACE="${PROJECT_NAME}-${env}-${cid}"
            	def imageName="${IMAG_NAME}"
            	def deployNamespace="${DEPLOY_NAMESPACE}"
            	def replicas="${params.REPLICAS}"
            	def svrName="${SVR_NAME}"
            	def appName="${APP_NAME}"
            	def port="${PORT}"
                def formattedAppName=APP_NAME.replaceAll("-", "")
                def logPath = "${ PROJECT_NAME }-${ formattedAppName }-${ params.ENVIRONMENT }-${ REGION }"
            	def monitor_port="16000"
            	def deploytpl="deploy/deploy.tpl"
                script {
                      sh """
                      
                      sed  '
                            
                            s#\$PROJECT_NAME#${projectName}#g
                            s#\$CID#${cid}#g
                            s#\$ENV#${env}#g
                            s#\$IMAGE#${imageName}#g
                            s#\$NAMESPACE#${deployNamespace}#g
                            s#\$REPLICAS#${replicas}#g
                            s#\$APP_NAME#${appName}#g
                            s#\$SVR_NAME#${svrName}#g
                            s#\$POD_PORT#${port}#g
                            s#\$MONITOR_PORT#${monitor_port}#g
                            s#\$CPU#${params.CPU}#g
                            s#\$MEMORY#${params.MEM}Gi#g
                            s#\$LOG_PATH#${logPath}#g
                            s#\$PHASE#release#g
                            ' ${deploytpl}> deploy/deploy.yml
                     
                            
                              echo "deploy文件如下："
                              cat deploy/deploy.yml
                            
                      """
                     }
                     
                def nstpl="deploy/namespace.tpl"
                script {
                      sh """
                      
                      sed  '
                            s#\$CID#${cid}#g
                            s#\$ENV#${env}#g
                            s#\$PROJECT_NAME#${PROJECT_NAME}#g
                            ' ${nstpl}> deploy/namespace.yml
                     
                              echo "namespace文件如下："
                              cat deploy/namespace.yml
                            
                      """
                     }
                def configMapYml = "deploy/configMap/${params.ENVIRONMENT}/SmartEnsembleOM-configMap.yml"
                script {
                    sh """
                    sed  -i '
                            s#\$NAMESPACE#${deployNamespace}#g
                            ' ${configMapYml}
                    """
                }

    }
    stage('Deploy'){

        def deployConfig = ""
        def deployConfigRelease = 'deploy/deploy.yml'
        def deployConfigCanary = 'deploy/deploy-canary.yml'
        def namespaceyml = 'deploy/namespace.yml'
        def configMapYml = "deploy/configMap/${params.ENVIRONMENT}/SmartEnsembleOM-configMap.yml"
        kubeConfigId = "${k8sEnv[params.ENVIRONMENT]}"
        if (!canary) {
            // delete canary release
            withCredentials([kubeconfigContent(credentialsId: "${kubeConfigId}",
                                               variable: 'kubeconfig')]) {
                sh """#!/bin/bash
                   mkdir -p ~/.kube/${params.ENVIRONMENT} && echo "\$kubeconfig" > ~/.kube/${params.ENVIRONMENT}/config
                   kubectl get ns ${NAMESPACE} --kubeconfig=/root/.kube/${params.ENVIRONMENT}/config
                   if [ \$? != 0 ]; then
                       kubectl create  -f  ${namespaceyml} --kubeconfig=/root/.kube/${params.ENVIRONMENT}/config
                   fi
                   kubectl -n ${NAMESPACE} get deployment "${APP_NAME}-canary" --kubeconfig=/root/.kube/${params.ENVIRONMENT}/config | grep $APP_NAME"-canary"
                   if [ \$? == 0 ]; then
                       kubectl -n ${NAMESPACE} delete deployment "${APP_NAME}-canary" --kubeconfig=/root/.kube/${params.ENVIRONMENT}/config >> /dev/null
                   fi
                   """
            }


            deployConfig = "${deployConfigRelease}"
        } else {
            deployConfig = "${deployConfigCanary}"
        }

        kubernetesDeploy(kubeconfigId: "$kubeConfigId",
                         configs: "${configMapYml}",
                         enableConfigSubstitution: true,
        )

        kubernetesDeploy(kubeconfigId: "$kubeConfigId",
                         configs: "${deployConfig}",
                         enableConfigSubstitution: true,
        )
    }
    stage("Check pod status"){
        withCredentials([kubeconfigContent(credentialsId: "${kubeConfigId}",
        variable: 'kubeconfig')]) {
            sh """#!/bin/bash
                   mkdir -p ~/.kube/${params.ENVIRONMENT} && echo "\$kubeconfig" > ~/.kube/${params.ENVIRONMENT}/config
                   message="There had some error in k8s deploy,plze contact ops to check problem"
                   sleep 30
                   kubectl -n $NAMESPACE rollout status deploy/$APP_NAME --timeout=60s --kubeconfig=/root/.kube/${params.ENVIRONMENT}/config
              """
        }
 
    }

}
