# -*- coding: utf-8 -*-
# !/usr/bin/env python3
# yule.wen 2021/7/8-3:04
# ceph super management script

import json
import sys
import os
import subprocess
import concurrent.futures
import datetime
from boto3.session import Session
import csv
import prettytable as pt

help_str = """
usage: python3 ceph_sre.py [action] [options...]
actions:
    bucket-upload [file_path] [bucket]                      Upload file to bucket              
    bucket-dir [bucket]                                     View the contents of the bucket
    bucket-delete [bucket]                                  Delete a bucket
    bucket-owner [bucket]                                   Query the owner of the bucket
    bucket-create [bucket] [owner]                          Create a bucket
    bucket-bullk-create [file_path] [owner]                 Create bucket from file
    bucket-policy [bucket,[all]]                            Query bucket permissions, [all] Export permissions matrix of all buckets
    bucket-count                                            Number of buckets 
    bucket-clean [bucket,[all]]                             Clean up the contents of the bucket, [all] Clean up all the contents of the bucket
    bucket-authorize [bucket] [auth_user] [permission]      Authorize a bucket to another user
    user-key [user,[all]]                                   Query a user’s secret key, [all] export a list of all users’ secret keys
    user-create [user]                                      Create a user
"""

date_obj = datetime.datetime.now()
date_str = "%s-%s-%s_%s:%s:%s" % (date_obj.year, date_obj.month, date_obj.day, date_obj.hour, date_obj.minute, date_obj.second)
from pathlib import Path
dir = Path('./output_policy')
if not dir.exists():
    os.mkdir('output_policy')

def get_role_key(user):
    cmd = 'radosgw-admin user info --uid=%s' % user
    role_key = json.load(os.popen(cmd)).get("keys")
    return role_key

def get_user_key_list(user):
    cmd = 'radosgw-admin user info --uid=%s' % user
    user_key_list = ['']*3
    key_dict = json.load(os.popen(cmd)).get("keys")[0]
    user_key_list[0] = key_dict["user"]
    user_key_list[1] = key_dict["access_key"]
    user_key_list[2] = key_dict["secret_key"]
    return user_key_list

def bucket_owner(bucket):
    cmd_str = "radosgw-admin bucket stats --bucket=%s" % bucket
    res = os.popen(cmd_str)
    owner = json.load(res).get("owner")
    return owner

def get_user_list():
    cmd_get_role_list = 'radosgw-admin user list'
    res_role_list = os.popen(cmd_get_role_list)
    role_list = json.load(res_role_list)
    return role_list

def exec_func(exec_str):
    res = subprocess.Popen(exec_str, shell=True,
                           stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    return str(res.stdout.read(), encoding='utf-8')

def upload_file_bucket(bucket, path):
    owner = bucket_owner(bucket)
    user_key = get_role_key(owner)
    access_key = user_key[0].get("access_key")
    secret_key = user_key[0].get("secret_key")

    exec_str = 's3cmd --no-ssl --access_key=%s --secret_key=%s --host-bucket= ' \
               '--host=127.0.0.1:7480 put %s s3://%s/' % (
                   access_key, secret_key, path, bucket)
    res = exec_func(exec_str)
    print(res)

def bucket_delete(bucket):

    owner = bucket_owner(bucket)
    user_key = get_role_key(owner)
    access_key = user_key[0].get("access_key")
    secret_key = user_key[0].get("secret_key")
    exec_str = 's3cmd --no-ssl --access_key=%s --secret_key=%s --host-bucket= ' \
               '--host=127.0.0.1:7480 rb s3://%s' % (
                   access_key, secret_key, bucket)

    res = exec_func(exec_str)
    print(res)

def bucket_dir(bucket):
    owner = bucket_owner(bucket)
    user_key = get_role_key(owner)
    access_key = user_key[0].get("access_key")
    secret_key = user_key[0].get("secret_key")
    exec_str = 's3cmd --no-ssl --access_key=%s --secret_key=%s --recursive --host-bucket= ' \
               '--host=127.0.0.1:7480 ls s3://%s/*' % (
                   access_key, secret_key, bucket)
    res = exec_func(exec_str)
    print(res)

def bucket_list():
    cmd_str = "radosgw-admin bucket list"
    res = json.load(os.popen(cmd_str))
    return res

def bucket_clean(bucket):
    owner = bucket_owner(bucket)
    user_key = get_role_key(owner)
    access_key = user_key[0].get("access_key")
    secret_key = user_key[0].get("secret_key")
    cmd_str = 's3cmd --no-ssl --access_key=%s --secret_key=%s --recursive --force --host-bucket= --host=127.0.0.1:7480 del s3://%s' %(access_key,secret_key,bucket)
    print(cmd_str)
    # cmd_res = exec_func(cmd_str) 
    # return cmd_res

def print_green(s):
    print('\033[0;32;40m %s \033[0m' % s)

def print_red(s):
    print('\033[0;31;40m %s \033[0m' % s)
    
def print_blue(s):
    print('\033[0;36;40m %s \033[0m' % s)

def get_bucket_policy(bucket):
    owner = bucket_owner(bucket)
    user_key = get_role_key(owner)
    access_key = user_key[0].get("access_key")
    secret_key = user_key[0].get("secret_key")
    url = "http://127.0.0.1:7480"
    session = Session(access_key, secret_key)
    s3_client = session.client('s3', endpoint_url=url)
    try:
        result = s3_client.get_bucket_policy(Bucket=bucket)
    except Exception as e:
        print("The bucket policy does not exist")
        return {}
    policy = json.loads(result['Policy'])

    if policy:
        return policy
    else:
        print("The bucket has no policy!\n New authorization matrix will be created！ ")
        return policy

def bucket_authorize(bucket,auth_user,permission):
    """[summary]

    Args:
        bucket ([type]): [description]
        auth_user ([type]): [description]
        permission ([type]): [description]

    Returns:
        [type]: [description]
    """
    date_obj = datetime.datetime.now()
    date_str = "%s-%s-%s_%s%s%s" % (
        date_obj.year, date_obj.month, date_obj.day, date_obj.hour, date_obj.month, date_obj.second)
    
    
    owner = bucket_owner(bucket)
    user_key = get_role_key(owner)
    access_key = user_key[0].get("access_key")
    secret_key = user_key[0].get("secret_key")
    user = auth_user
    
    def police_template(auth_user, auth_action, auth_bucket):
        return {
            "Sid": auth_user + '.' + auth_action,
            "Effect": "Allow",
            "Principal": {"AWS": "arn:aws:iam:::user/" + auth_user},
            "Action": auth_action,
            "Resource": [
                "arn:aws:s3:::" + auth_bucket + "/*",
                "arn:aws:s3:::" + auth_bucket,
            ]
        }    
    statements = []
    def auth_temp(auth_policy_type):
        if auth_policy_type == "read":
            statements.append(police_template(user, "s3:GetObject*", bucket))
            statements.append(police_template(user, "s3:ListBucket*", bucket))
        elif auth_policy_type == "write":
            statements.append(police_template(user, "s3:DeleteObject*", bucket))
            statements.append(police_template(user, "s3:PutObject*", bucket))
        elif auth_policy_type == "all":
            statements.append(police_template(user, "s3:*", bucket))
        else:
            print_red("auth error")
            exit(1)
        return statements

    policy_list = auth_temp(permission)    
    old_policy = get_bucket_policy(bucket)
    
    if old_policy:
        old_list = old_policy.get("Statement")
        for i in policy_list:
            old_list.append(i)
        police = {"Version": "2012-10-17", "Statement": old_list}
    else:
        police = {"Version": "2012-10-17", "Statement": policy_list}
    from pathlib import Path
    dir = Path('./policy_record')
    if not dir.exists():
        os.mkdir('policy_record')
    police_save_filename = "./policy_record/%s_policy_%s_%s.json" % (bucket, permission, date_str)
    with open(police_save_filename, 'a+') as f:
        f.write(json.dumps(police))
        exec_str = "s3cmd --no-ssl --access_key=%s --secret_key=%s --host-bucket= --host=127.0.0.1:7480 setpolicy %s " \
                   "s3://%s" % (
                       access_key, secret_key, police_save_filename, bucket)    
    res = exec_func(exec_str)
    print(res)
    print_green("auth policy success")

def bucket_create(bucket,owner):
    try:
        user_key = get_role_key(owner)
    except Exception as e:
        print_red("the user is not exist")
    access_key = user_key[0].get("access_key")
    secret_key = user_key[0].get("secret_key")
    exec_str = 's3cmd --no-ssl --access_key=%s --secret_key=%s --host-bucket= ' \
               '--host=127.0.0.1:7480 mb s3://%s' % (access_key, secret_key, bucket)
    res = exec_func(exec_str)
    print_green(res)

def mutiprocess_exec(func,args_list):
    with concurrent.futures.ThreadPoolExecutor() as executor:
        res = executor.map(func,args_list)       
        return res
    
def bucket_policy(bucket):
    owner = bucket_owner(bucket)
    bucket_policy = get_bucket_policy(bucket)
    # print('bucket_policy-----',bucket_policy)
    policy_matrix = []
    policy_header = ['bucket','owner','read_user','write_user','all_right_user']
    policy_list = ['']*5
    read_right = ("s3:GetObject*","s3:ListBucket*")
    write_right = ("s3:DeleteObject*","s3:PutObject*")
    all_right = ("s3:*")
    if bucket_policy: # has policy
        policy_list[0] = bucket
        policy_list[1] = owner
        statement_list = bucket_policy["Statement"]
        # print('statement_list----55',statement_list)
        read_user = []
        write_user = []
        all_right_user = []
        for state in statement_list:
            print('state["Principal"]["AWS"]',state["Principal"]["AWS"])
            user = state["Principal"]["AWS"].split('/')[-1]
            # user = state["Principal"]["AWS"]
            action = state["Action"]
            if action in read_right and user not in read_user:
                read_user.append(user)
            elif action in write_right and user not in write_user:
                write_user.append(user)
            elif action in all_right and user not in all_right_user:
                all_right_user.append(user)
            else:
                print("Action ERROR")
        # print('read_user %s write_user %s '%(read_user,write_user))
        policy_list[2] = ",".join(read_user)
        policy_list[3] = ",".join(write_user)
        policy_list[4] = ",".join(all_right_user)
    else:
        # has no plicy
        policy_list[0] = bucket
        policy_list[1] = owner
        
    # print('policy_list---',policy_list)
    return policy_list

def user_create(user):
    cmd_str = "radosgw-admin user create --uid=%s --display-name=%s " %(user,user)
    res = os.popen(cmd_str)
    print(json.load(res))
    
args_list = sys.argv
args_lens = len(args_list)
if args_lens < 2:
    print(help_str)
    exit(1)
else:
    action = args_list[1]
    
    if action == 'bucket-upload':
        """[summary]
        """
        file_path = args_list[2]
        bucket = args_list[3] 
        upload_file_bucket(bucket, file_path)

    elif action == 'bucket-dir':
        bucket = args_list[2].strip()
        bucket_dir(bucket)

    elif action == 'bucket-delete':
        bucket = args_list[2].strip()
        bucket_delete(bucket)

    elif action == 'bucket-owner':
        bucket = args_list[2].strip()
        res = bucket_owner(bucket)
        print(res)

    elif action == 'bucket-create':
        """
        """
        bucket = args_list[2].strip()
        owner = args_list[3].strip()
        bucket_create(bucket,owner)

    elif action == 'bucket-bullk-create':
        path = args_list[2].strip()
        try:
            with open(path, 'r') as f:
                bucket_name_list = f.readlines()
                print(bucket_name_list)
        except Exception as e:
            print_red("file not exist")
        owner = args_list[3].strip()
        with concurrent.futures.ThreadPoolExecutor() as executor:
            to_do = []
            for bk in bucket_name_list:
                future = executor.submit(bucket_create,*(bk,owner))
                to_do.append(future)
            for future in concurrent.futures.as_completed(to_do):
                future.result()
        
    elif action == 'bucket-policy':
        """
        output the matrix of the bucket
        """
        date_obj = datetime.datetime.now()
        date_str = "%s-%s-%s_%s%s%s" % (
        date_obj.year, date_obj.month, date_obj.day, date_obj.hour, date_obj.month, date_obj.second)
        policy_matrix = []
        policy_header = ['bucket','owner','read_user','write_user','all_right_user']
        policy_matrix.append(policy_header)
        bucket = args_list[2].strip()
        if bucket == 'all':
            # export all bucket policy
            output_policy_filename = "./output_policy/%s_policy_output_%s.csv" % ('all_bucket', date_str)
            bk_list = bucket_list()
            with concurrent.futures.ThreadPoolExecutor() as executor:
                to_do = []
                for bk in bk_list:
                    future = executor.submit(bucket_policy,bk)
                    to_do.append(future)
                for future in concurrent.futures.as_completed(to_do):
                    res = future.result()
                    policy_matrix.append(res)
                print(policy_matrix)
        else:
            output_policy_filename = "./output_policy/%s_policy_output_%s.csv" % (bucket, date_str)
            res = bucket_policy(bucket)
            policy_matrix.append(res)
            print(policy_matrix)

        with open (output_policy_filename,'w') as f:
            f_csv = csv.writer(f)
            f_csv.writerows(policy_matrix)
            print_blue("The bucket policy matrix file output: %s .you can download it" %output_policy_filename)
            
        tb = pt.PrettyTable()
        tb.field_names = policy_matrix[0]
        tb.align["bucket"] = "l"
        tb.align["owner"] = "l"
        for row in policy_matrix[1:]:
            tb.add_row(row)
        tb.set_style(pt.DEFAULT)
        print_green(tb)
            
    elif action == 'bucket-count':
        """[summary]
        """
        bk_list = bucket_list()
        # print(bk_list)
        print('The total number of buckets: %s' % len(bk_list))

    elif action == 'bucket-clean':
        """ 
        clean bucket files, will delete all file in bucket !
        """
        bucket = args_list[2].strip()
        if bucket == 'all':
            # 多线程启动
            bk_list = bucket_list()
            mutiprocess_exec(bucket_clean,bk_list)
        else:   
            res = bucket_clean(bucket)
            print(res)
        
    elif action == 'bucket-authorize':
        """
        authorize a bucket to other user account 
        """
        bucket = args_list[2].strip() 
        auth_user = args_list[3].strip()
        permission = args_list[4].strip()
        bucket_authorize(bucket,auth_user,permission)

    elif action == 'user-key':
        user = args_list[2].strip()
        
        user_key_list = [["user_name","access_key","secret_key"],['--','--','update time:'+date_str],]
        
        if user == 'all':
            output_users_filename = "./output_policy/%s_output_%s.csv" % ('all_users_key', date_str)
            user_list = get_user_list()
            with concurrent.futures.ThreadPoolExecutor() as executor:
                to_do = []
                for user in user_list:
                    future = executor.submit(get_user_key_list,user)
                    to_do.append(future)
                for future in concurrent.futures.as_completed(to_do):
                    res = future.result()
                    user_key_list.append(res)
                # print(policy_matrix)       
            with open (output_users_filename,'w') as f:
                f_csv = csv.writer(f)
                f_csv.writerows(user_key_list)
                print_blue("The all user key file output: %s .You can download it !" %output_users_filename)        
        else:   
            res = get_user_key_list(user)
            user_key_list.append(res)
        
        tb = pt.PrettyTable()
        tb.field_names = user_key_list[0]
        tb.align["user_name"] = "l"
        # tb.align["access_key"] = "l"
        for row in user_key_list[1:]:
            tb.add_row(row)
        tb.set_style(pt.DEFAULT)
        print_green(tb)   
    elif action == 'user-create':
        user = args_list[2].strip()
        res = user_create(user)
        print("create success！")
    else:
        print("plz input correct!")
        print(help_str)
        exit(1)